import re
import percy

class UserAgent(object):

    def __init__(self, config, environment):
      self.config = config
      self.environment = environment

    def __str__(self):
        client = ' '.join(filter(None, [
          "Percy/%s" % self._api_version(),
          "python-percy-client/%s" % self._client_version(),
        ]))

        environment = '; '.join(filter(None, [
          self._environment_info(),
          "python/%s" % self._python_version(),
          self.environment.current_ci,
        ]))

        return "%s (%s)" % (client, environment)

    def _client_version(self):
        return percy.__version__

    def _python_version(self):
        try:
            from platform import python_version
            return python_version()
        except ImportError:
            return 'unknown'

    def _django_version(self):
        try:
            import django
            return "django/%s" % django.get_version()
        except ImportError:
            return None

    def _api_version(self):
        return re.search('\w+$', self.config.api_url).group(0)

    def _environment_info(self):
        # we only detect django right now others could be added
        return self._django_version()



import os
import requests_mock
import unittest
import percy
from percy import connection


class TestPercyConnection(unittest.TestCase):
    def setUp(self):
        config = percy.Config(access_token='foo')
        environment = percy.Environment()

        self.percy_connection = connection.Connection(config, environment)

    @requests_mock.Mocker()
    def test_get(self, mock):
        mock.get('http://api.percy.io', text='{"data":"GET Percy"}')
        data = self.percy_connection.get('http://api.percy.io')
        self.assertEqual(data['data'], 'GET Percy')

        auth_header = mock.request_history[0].headers['Authorization']
        assert auth_header == 'Token token=foo'

    @requests_mock.Mocker()
    def test_post(self, mock):
        mock.post('http://api.percy.io', text='{"data":"POST Percy"}')
        data = self.percy_connection.post(
            'http://api.percy.io',
            data='{"data": "data"}'
        )
        self.assertEqual(data['data'], 'POST Percy')

        auth_header = mock.request_history[0].headers['Authorization']
        assert auth_header == 'Token token=foo'

    # @requests_mock.Mocker()
    # def test_post_error(self, mock):
    #     mock.post('http://api.percy.io', text='{"error":"foo"}', status_code=500)
    #     try:
    #         data = self.percy_connection.post(
    #             'http://api.percy.io',
    #             data='{"data": "data"}'
    #         )
    #     except:
    #         return
    #     else:
    #         raise Exception('Error not raised')



import os
import unittest

from percy import errors
from percy import config


class TestPercyConfig(unittest.TestCase):
    def setUp(self):
        os.environ['PERCY_TOKEN'] = 'abcd1234'
        self.config = config.Config()

    def tearDown(self):
        del os.environ['PERCY_TOKEN']

    def test_getters(self):
        self.assertEqual(self.config.api_url, 'https://percy.io/api/v1')
        self.assertEqual(self.config.default_widths, [])
        self.assertEqual(self.config.access_token, 'abcd1234')

    def test_setters(self):
        self.config.api_url = 'https://microsoft.com/'
        self.assertEqual(self.config.api_url, 'https://microsoft.com/')

        self.assertEqual(self.config.default_widths, [])
        self.config.default_widths = (1280, 375)
        self.assertEqual(self.config.default_widths, (1280, 375))

        self.config.access_token = None
        self.assertRaises(errors.AuthError, lambda: self.config.access_token)

        self.config.access_token = 'percy123'
        self.assertEqual(self.config.access_token, 'percy123')



# -*- coding: utf-8 -*-

import os
import sys
import unittest

from percy import utils


class TestPercyUtils(unittest.TestCase):
    def test_sha256hash(self):
        self.assertEqual(
          utils.sha256hash('foo'),
          '2c26b46b68ffc68ff99b453c1d30413413422d706483bfa0f98a5e886266e7ae',
        )
        self.assertEqual(
          utils.sha256hash(u'I ♡ Python'),
          '5ebc381ed49ffbbff4efabd2c5e2ac5f116984cd72fffa4272cc528176bb09f6',
        )

        if sys.version_info >= (3,0):
          binary_content = b'\x01\x02\x99'
        else:
          binary_content = '\x01\x02\x99'

        self.assertEqual(
          utils.sha256hash(binary_content),
          '17b9440d8fa6bb5eb0e06a68dd3882c01c5a757f889118f9fbdf50e6e7025581',
        )

    def test_base64encode(self):
        self.assertEqual(utils.base64encode('foo'), 'Zm9v')
        self.assertEqual(utils.base64encode(u'I ♡ Python'), 'SSDimaEgUHl0aG9u')

        if sys.version_info >= (3,0):
          binary_content = b'\x01\x02\x99'
        else:
          binary_content = '\x01\x02\x99'

        self.assertEqual(utils.base64encode(binary_content), 'AQKZ')



import os
import re
import subprocess

from percy import errors
from percy import utils

__all__ = ['Environment']


class Environment(object):
    def __init__(self):
        self._real_env = None
        if os.getenv('TRAVIS_BUILD_ID'):
            self._real_env = TravisEnvironment()
        elif os.getenv('JENKINS_URL'):
            self._real_env = JenkinsEnvironment()
        elif os.getenv('CIRCLECI'):
            self._real_env = CircleEnvironment()
        elif os.getenv('CI_NAME') == 'codeship':
            self._real_env = CodeshipEnvironment()
        elif os.getenv('DRONE') == 'true':
            self._real_env = DroneEnvironment()
        elif os.getenv('SEMAPHORE') == 'true':
            self._real_env = SemaphoreEnvironment()
        elif os.getenv('BUILDKITE') == 'true':
            self._real_env = BuildkiteEnvironment()

    @property
    def current_ci(self):
        if self._real_env:
            return self._real_env.current_ci

    @property
    def pull_request_number(self):
        if os.getenv('PERCY_PULL_REQUEST'):
            return os.getenv('PERCY_PULL_REQUEST')
        if self._real_env and hasattr(self._real_env, 'pull_request_number'):
            return self._real_env.pull_request_number

    @property
    def branch(self):
        # First, percy env var.
        if os.getenv('PERCY_BRANCH'):
            return os.getenv('PERCY_BRANCH')
        # Second, from the CI environment.
        if self._real_env and hasattr(self._real_env, 'branch'):
            return self._real_env.branch
        # Third, from the local git repo.
        raw_branch_output = self._raw_branch_output()
        if raw_branch_output:
            return raw_branch_output
        # Fourth, fallback to 'master'.
        utils.print_error('[percy] Warning: unknown git repo, setting PERCY_BRANCH to "master".')
        return 'master'

    def _raw_branch_output(self):
        process = subprocess.Popen(
            'git rev-parse --abbrev-ref HEAD 2> /dev/null', stdout=subprocess.PIPE, shell=True)
        return process.stdout.read().strip().decode('utf-8')

    def _get_origin_url(self):
        process = subprocess.Popen(
            'git config --get remote.origin.url', stdout=subprocess.PIPE, shell=True)
        return process.stdout.read().strip().decode('utf-8')

    @property
    def commit_sha(self):
        # First, percy env var.
        if os.getenv('PERCY_COMMIT'):
            return os.getenv('PERCY_COMMIT')
        # Second, from the CI environment.
        if self._real_env and hasattr(self._real_env, 'commit_sha'):
            return self._real_env.commit_sha

    @property
    def repo(self):
        if os.getenv('PERCY_REPO_SLUG') or os.getenv('PERCY_PROJECT'):
            return os.getenv('PERCY_REPO_SLUG') or os.getenv('PERCY_PROJECT')
        if self._real_env and hasattr(self._real_env, 'repo'):
            return self._real_env.repo

        origin_url = self._get_origin_url()
        if not origin_url:
          raise errors.RepoNotFoundError(
            'No local git repository found. ' +
            'You can manually set PERCY_PROJECT=org/repo-name to fix this.')

        match = re.compile(r'.*[:/]([^/]+\/[^/]+?)(\.git)?\Z').match(origin_url)
        if not match:
          raise errors.RepoNotFoundError(
            "Could not determine repository name from URL: {0}\n" +
            "You can manually set PERCY_PROJECT=org/repo-name to fix this.".format(origin_url))
        return match.group(1)

    @property
    def parallel_nonce(self):
        if os.getenv('PERCY_PARALLEL_NONCE'):
          return os.getenv('PERCY_PARALLEL_NONCE')
        if self._real_env and hasattr(self._real_env, 'parallel_nonce'):
            return self._real_env.parallel_nonce

    @property
    def parallel_total_shards(self):
        if os.getenv('PERCY_PARALLEL_TOTAL'):
            return int(os.getenv('PERCY_PARALLEL_TOTAL'))
        if self._real_env and hasattr(self._real_env, 'parallel_total_shards'):
            return self._real_env.parallel_total_shards


class TravisEnvironment(object):
    @property
    def current_ci(self):
        return 'travis'

    @property
    def pull_request_number(self):
        pr_num = os.getenv('TRAVIS_PULL_REQUEST')
        if pr_num != 'false':
            return pr_num

    @property
    def branch(self):
        if self.pull_request_number and os.getenv('TRAVIS_PULL_REQUEST_BRANCH'):
            return os.getenv('TRAVIS_PULL_REQUEST_BRANCH')
        return os.getenv('TRAVIS_BRANCH')

    @property
    def repo(self):
        return os.getenv('TRAVIS_REPO_SLUG')

    @property
    def commit_sha(self):
        if self.pull_request_number and os.getenv('TRAVIS_PULL_REQUEST_SHA'):
            return os.getenv('TRAVIS_PULL_REQUEST_SHA')
        return os.getenv('TRAVIS_COMMIT')

    @property
    def parallel_nonce(self):
        return os.getenv('TRAVIS_BUILD_NUMBER')

    @property
    def parallel_total_shards(self):
        if os.getenv('CI_NODE_TOTAL', '').isdigit():
            return int(os.getenv('CI_NODE_TOTAL'))


class JenkinsEnvironment(object):
    @property
    def current_ci(self):
        return 'jenkins'

    @property
    def pull_request_number(self):
        # GitHub Pull Request Builder plugin.
        return os.getenv('ghprbPullId')

    @property
    def branch(self):
        return os.getenv('ghprbSourceBranch')

    @property
    def commit_sha(self):
        return os.getenv('ghprbActualCommit') or os.getenv('GIT_COMMIT')

    @property
    def parallel_nonce(self):
        return os.getenv('BUILD_NUMBER')


class CircleEnvironment(object):
    @property
    def current_ci(self):
        return 'circle'

    @property
    def pull_request_number(self):
        pr_url = os.getenv('CI_PULL_REQUEST')
        if pr_url:
          return os.getenv('CI_PULL_REQUEST').split('/')[-1]

    @property
    def branch(self):
        return os.getenv('CIRCLE_BRANCH')

    @property
    def repo(self):
        return "{0}/{1}".format(
            os.getenv('CIRCLE_PROJECT_USERNAME'),
            os.getenv('CIRCLE_PROJECT_REPONAME'),
        )

    @property
    def commit_sha(self):
        return os.getenv('CIRCLE_SHA1')

    @property
    def parallel_nonce(self):
        return os.getenv('CIRCLE_BUILD_NUM')

    @property
    def parallel_total_shards(self):
        if os.getenv('CIRCLE_NODE_TOTAL', '').isdigit():
            return int(os.getenv('CIRCLE_NODE_TOTAL'))


class CodeshipEnvironment(object):
    @property
    def current_ci(self):
        return 'codeship'

    @property
    def pull_request_number(self):
        pr_num = os.getenv('CI_PULL_REQUEST')
        # Unfortunately, codeship seems to always returns 'false', so let this be null.
        if pr_num != 'false':
          return pr_num

    @property
    def branch(self):
        return os.getenv('CI_BRANCH')

    @property
    def commit_sha(self):
        return os.getenv('CI_COMMIT_ID')

    @property
    def parallel_nonce(self):
        return os.getenv('CI_BUILD_NUMBER')

    @property
    def parallel_total_shards(self):
        if os.getenv('CI_NODE_TOTAL', '').isdigit():
            return int(os.getenv('CI_NODE_TOTAL'))


class DroneEnvironment(object):
    @property
    def current_ci(self):
        return 'drone'

    @property
    def pull_request_number(self):
        return os.getenv('CI_PULL_REQUEST')

    @property
    def branch(self):
        return os.getenv('DRONE_BRANCH')

    @property
    def commit_sha(self):
        return os.getenv('DRONE_COMMIT')


class SemaphoreEnvironment(object):
    @property
    def current_ci(self):
        return 'semaphore'

    @property
    def pull_request_number(self):
        return os.getenv('PULL_REQUEST_NUMBER')

    @property
    def branch(self):
        return os.getenv('BRANCH_NAME')

    @property
    def repo(self):
      return os.getenv('SEMAPHORE_REPO_SLUG')

    @property
    def commit_sha(self):
        return os.getenv('REVISION')

    @property
    def parallel_nonce(self):
        return os.getenv('SEMAPHORE_BUILD_NUMBER')

    @property
    def parallel_total_shards(self):
        if os.getenv('SEMAPHORE_THREAD_COUNT', '').isdigit():
            return int(os.getenv('SEMAPHORE_THREAD_COUNT'))


class BuildkiteEnvironment(object):
    @property
    def current_ci(self):
        return 'buildkite'

    @property
    def pull_request_number(self):
        return os.getenv('BUILDKITE_PULL_REQUEST')

    @property
    def branch(self):
        return os.getenv('BUILDKITE_BRANCH')

    @property
    def commit_sha(self):
        if os.getenv('BUILDKITE_COMMIT') == 'HEAD':
            # Buildkite mixes SHAs and non-SHAs in BUILDKITE_COMMIT, so we return null if non-SHA.
            return
        return os.getenv('BUILDKITE_COMMIT')

    @property
    def parallel_nonce(self):
        return os.getenv('BUILDKITE_BUILD_ID')

    @property
    def parallel_total_shards(self):
        if os.getenv('BUILDKITE_PARALLEL_JOB_COUNT', '').isdigit():
            return int(os.getenv('BUILDKITE_PARALLEL_JOB_COUNT'))



import unittest
import re
import percy
from percy.user_agent import UserAgent

class TestPercyUserAgent(unittest.TestCase):
    def setUp(self):
        self.config = percy.Config(access_token='foo')
        self.environment = percy.Environment()

    def test_string(self):
        self.assertTrue(
            re.match(
                '^Percy/v1 python-percy-client/[.\d]+ \(python/[.\d]+(; travis)?\)$',
                str(UserAgent(self.config, self.environment))
            )
        )



# -*- coding: utf-8 -*-

__author__ = 'Perceptual Inc.'
__email__ = 'team@percy.io'
__version__ = '1.0.3'

from percy.client import *
from percy.config import *
from percy.environment import *
from percy.resource import *
from percy.resource_loader import *
from percy.runner import *



# -*- coding: utf-8 -*-

from percy.connection import Connection
from percy.environment import Environment
from percy.config import Config
from percy import utils

__all__ = ['Client']


class Client(object):

    def __init__(self, connection=None, config=None, environment=None):
        self._environment = environment if environment else Environment()
        self._config = config if config else Config()
        self._connection = connection if connection else Connection(self._config, self._environment)

    @property
    def connection(self):
        return self._connection

    @property
    def config(self):
        return self._config

    @property
    def environment(self):
        return self._environment

    def create_build(self, **kwargs):
        repo = kwargs.get('repo') or self.environment.repo
        branch = kwargs.get('branch') or self.environment.branch
        commit_sha = kwargs.get('commit_sha') or self.environment.commit_sha
        pull_request_number = kwargs.get('pull_request_number') \
            or self.environment.pull_request_number
        resources = kwargs.get('resources')
        parallel_nonce = kwargs.get('parallel_nonce') or self.environment.parallel_nonce
        parallel_total_shards = kwargs.get('parallel_total_shards') \
            or self.environment.parallel_total_shards

        # Only pass parallelism data if it all exists.
        if not parallel_nonce or not parallel_total_shards:
            parallel_nonce = None
            parallel_total_shards = None

        data = {
            'data': {
                'type': 'builds',
                'attributes': {
                    'branch': branch,
                    'commit-sha': commit_sha,
                    'pull-request-number': pull_request_number,
                    'parallel-nonce': parallel_nonce,
                    'parallel-total-shards': parallel_total_shards,
                }
            }
        }

        if resources:
            data['data']['relationships'] = {
                'resources': {
                    'data': [r.serialize() for r in resources],
                }
            }

        path = "{base_url}/repos/{repo}/builds/".format(
            base_url=self.config.api_url,
            repo=repo,
        )
        return self._connection.post(path=path, data=data)

    def finalize_build(self, build_id):
        path = "{base_url}/builds/{build_id}/finalize".format(
            base_url=self.config.api_url,
            build_id=build_id,
        )
        return self._connection.post(path=path, data={})

    def create_snapshot(self, build_id, resources, **kwargs):
        if not resources or len(resources) <= 0:
            raise ValueError(
                'resources should be an array of Percy.Resource objects'
            )
        widths = kwargs.get('widths', self.config.default_widths)
        data = {
            'data': {
                'type': 'snapshots',
                'attributes': {
                    'name': kwargs.get('name'),
                    'enable-javascript': kwargs.get('enable_javascript'),
                    'widths': widths,
                },
                'relationships': {
                    'resources': {
                        'data': [r.serialize() for r in resources],
                    }
                }
            }
        }
        path = "{base_url}/builds/{build_id}/snapshots/".format(
            base_url=self.config.api_url,
            build_id=build_id
        )
        return self._connection.post(path=path, data=data)

    def finalize_snapshot(self, snapshot_id):
        path = "{base_url}/snapshots/{snapshot_id}/finalize".format(
            base_url=self.config.api_url,
            snapshot_id=snapshot_id
        )
        return self._connection.post(path=path, data={})

    def upload_resource(self, build_id, content):
        sha = utils.sha256hash(content)
        data = {
            'data': {
                'type': 'resources',
                'id': sha,
                'attributes': {
                    'base64-content': utils.base64encode(content),
                }

            }
        }
        path = "{base_url}/builds/{build_id}/resources/".format(
            base_url=self.config.api_url,
            build_id=build_id
        )
        return self._connection.post(path=path, data=data)



from __future__ import print_function
import sys
import hashlib
import base64

# TODO: considering using the 'six' library here, but for now just do something simple.

def print_error(*args, **kwargs):
    print(*args, file=sys.stderr, **kwargs)

def sha256hash(content):
    if _is_unicode(content):
        content = content.encode('utf-8')
    return hashlib.sha256(content).hexdigest()

def base64encode(content):
    if _is_unicode(content):
        content = content.encode('utf-8')
    return base64.b64encode(content).decode('utf-8')

def _is_unicode(content):
    if (sys.version_info >= (3,0) and isinstance(content, str)
        or sys.version_info < (3,0) and isinstance(content, unicode)):
        return True
    return False
